;*********************************************************************
; 
; Purpose: Demo code showing how to move through a database with and
;          without an index. Without an index, movement through the
;          database is based on the order the records are physically
;          stored in the database. If an index is used then the
;          database records are displayed based on the sort order
;          of the keys in the index.
; 
; Paul Squires (2000-2003)
; 
;*********************************************************************

 
;Define the names of the database & index
    DBFname.s = "Cheetah.dbf"
    IDXname.s = "Cheetah.idx"

;Create the database
    AllFields.s = "CUSTID,C,7,0;CUSTNAME,C,35,0;SALARY,N,14,2"
    xdbCreate(DBFName, AllFields)
 
;Open the database (database must be open prior To creating index)
    dbHandle.l = xdbOpen(DBFName, "") ; no encryption = ""
    Debug xdbAnyErrors() 

;Create the index (database must be open)
   IndexExpr.s = "UPPER(CUSTID)"   ;index is not case sensitive
   Duplicates.l = #XDBTRUE         ;allow duplicate customer ID's
    
   xdbCreateIndex(IDXName, dbHandle, IndexExpr, Duplicates)

;Open the index
   idxHandle.l = xdbOpenIndex(IDXname, dbHandle)
 
   msg.s = "DBFname: " + DBFname + Chr(10) + "dbHandle:" +  Str(dbHandle) + Chr(10) + Chr(10) + "IDXname: " + IDXname + Chr(10) + "idxHandle:" + Str(idxHandle)
   MessageRequester("", msg, 0)

;Add records to the database and index
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;NOTE: When adding a large number of records it is better to close
;the index, do the appends to the database and then open the
;index and reindex. For a few hundred records there should be no
;performance issues.

NumRecs.l = 200

Msg.s = "Press 'Okay' To create " + Str(NumRecs) + " database and index these records."
MessageRequester("PureBasic",msg, 0)

; ------------------------------------------------------------------------
; Predefine the field#'s for speed. If you already know the
; which field name corresponds to which field number then
; there is no need to do this. These functions provide
; really fast lookups.
;------------------------------------------------------------------------

   CustIDfield.l = xdbFieldNumber(dbHandle, "CUSTID")
   CustNameField.l = xdbFieldNumber(dbHandle,"CUSTNAME")
   SalaryField.l = xdbFieldNumber(dbHandle, "SALARY")
  
  For x.l = 1 To NumRecs

      xdbClearBuffer(dbHandle)  ;this will clear the record buffer
     ;NOTE: You must do this step to assure clean information
          
      CustID.s = RSet(Trim(Str(x)), 7)
           
      xdbAssignField(dbHandle, "", CustIDfield, CustID)
      xdbAssignField(dbHandle, "", CustNameField, "The big Cat " + Str(x))
      xdbAssignField(dbHandle, "", SalaryField, "-1234.50")
      
     ;Add to the end of the database (Append) & add the key to the index.
      xdbAddRecord(dbHandle)  
      
  Next x

;Close the database and related index
 xdbClose(dbHandle)                                 
  

MessageRequester("PureBasic","Database and Index records created.", 0)    
End











                                                                                                         
; ExecutableFormat=Windows
; CursorPosition=13
; FirstLine=1
; EOF