;*********************************************************************
; 
; Purpose: Demo code showing how to add and delete memo records. Memo 
;          fields are created using the "M" or "Z" field types. The 
;          actual memo data is stored in a separate file. For this
;          reason it is more efficient for you to store smaller strings
;          in "C" character fields. Those fields can store strings up
;          to 256 characters in length. Any strings longer than that 
;          will have to be stored in memo fields.
; 
; Paul Squires (2000-2003)
; 
;*********************************************************************


  Title.s = "PureBasic Cheetah Database"

; Define the names of the database & index
    DBFname.s = "Cheetah.dbf"
    IDXname.s = "Cheetah.idx"

; Build the same as the above file definition.
    AllFields.s = "CUSTID,C,7,0;CUSTNAME,C,35,0;SALARY,N,14,2;NOTES,M,10,0"

; Create the database
     xdbCreate(DBFname, AllFields) 
;   to create a database using encyption and/or variable length memo fields then 
;   use the xdbCreateEX function.
 
;Open the database (database must be open prior To creating index)
   dbHandle = xdbOpen(DBFname, "") ; no encryption
 
;Create the index (database must be open)
    IndexExpr.s = "UPPER(CUSTID)"  ;index is not case sensitive
    Duplicates.l = #XDBTRUE
    xdbCreateIndex(IDXname, dbHandle, IndexExpr, Duplicates)

    
;Open the index
     idxHandle = xdbOpenIndex(IDXname, dbHandle)
 
; Add records to the database and index
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
NumRecs.l = 200

Msg.s = "Press 'Okay' To create " + Str(NumRecs) + " database and index these records."
MessageRequester("PureBasic",msg, 0)

; ------------------------------------------------------------------------
; Predefine the field#'s for speed. If you already know the
; which field name corresponds to which field number then
; there is no need to do this. These functions provide
; really fast lookups.
; ------------------------------------------------------------------------
  CustIDfield = xdbFieldNumber(dbHandle, "CUSTID")
  CustNameField = xdbFieldNumber(dbHandle, "CUSTNAME")
  SalaryField = xdbFieldNumber(dbHandle, "SALARY")
  NotesField = xdbFieldNumber(dbHandle, "NOTES")

   

  For x.l = 1 To NumRecs

      xdbClearBuffer(dbHandle)   ;This clears the record buffer
                                 ;NOTE: You must do this step to assure clean information
     ;RSet CustID$ = LTrim$(Str$(x&))
      CustID.s = RSet(Trim(Str(x)), 7)
           
      xdbAssignField(dbHandle, "", CustIDfield, CustID)
      xdbAssignField(dbHandle, "", CustNameField, "Billy Gates" + Str(x))
      xdbAssignField(dbHandle, "", SalaryField, "100234.50")
      xdbAssignField(dbHandle, "", NotesField, "Hi Bob")
 
     ;Add to the end of the database (Append) & add the key to the index automatically
      xdbAddRecord(dbHandle)  
      xdbAnyErrors()
  Next x

; Modify rec #100 by changing the notes memo attached to it.
    Notes.s = "This is inside the memo field in record #100."
    
    xdbGetRecord(dbHandle, 100)
    xdbAssignField(dbHandle, "", NotesField, Notes)

;Store the information
    PutRecord.l = 100
    xdbPutRecord(dbHandle, PutRecord)
    xdbAnyErrors()

; Now find and display the note
    xdbGetRecord(dbHandle, 100)
  
    Notes.s = xdbFieldValue(dbHandle, "", NotesField) ; <-------------------------
   ;Notes.s = xdbFieldValue(dbHandle, "", 4)  ; <----- you can use either above method or this one.

    Msg.s  ="Memo for Record# 100: " +  Notes
    MessageRequester(Title, msg, 0)

; Delete the note (this is accomplished by writing "" to the memo field.
    xdbGetRecord(dbHandle, 100)
    xdbAssignField(dbHandle, "", NotesField, "")

;Store the information
    xdbPutRecord(dbHandle, 100)
    xdbAnyErrors()
   
; Ddisplay the note (it should not exist)
    xdbGetRecord(dbHandle, 100)
   
    Notes.s = xdbFieldValue(dbHandle, "", NotesField)
    Msg.s  ="Update Memo (deleted) for Record# 100: " + Notes
    MessageRequester(Title, msg, 0)

;Close the database And related index
    xdbClose(dbHandle)                                 
    Msg.s  ="Completed"
    MessageRequester(Title, msg, 0)


End


; ExecutableFormat=Windows
; CursorPosition=13
; FirstLine=1
; EOF