;*********************************************************************
; 
; Purpose: Demo code showing how to move through a database with and
;          without an index. Without an index, movement through the
;          database is based on the order the records are physically
;          stored in the database. If an index is used then the
;          database records are displayed based on the sort order
;          of the keys in the index.
; 
; Paul Squires (2000-2003)
; 
;*********************************************************************

;Define the names of the database & index
    DBFname.s = "Cheetah.dbf"
    IDXname.s = "Cheetah.idx"

Title.s = "PureBasic Cheetah Database"  ;   Title$ = "PowerBasic Cheetah Database"
;Define the names of the database & index
    DBFname.s = "Cheetah.dbf"
    IDXname.s = "Cheetah.idx"

;Create the database
    AllFields.s = "CUSTID,C,7,0;CUSTNAME,C,35,0;SALARY,N,14,2"
    xdbCreate(DBFName, AllFields)
 
;Open the database (database must be open prior To creating index)
    dbHandle.l = xdbOpen(DBFName, "") ; no encryption = ""
    Debug xdbAnyErrors() 

;Create the index (database must be open)
   IndexExpr.s = "UPPER(CUSTID)"   ;index is not case sensitive
   Duplicates.l = #XDBTRUE         ;allow duplicate customer ID's
    
   xdbCreateIndex(IDXName, dbHandle, IndexExpr, Duplicates)

;Open the index
   idxHandle.l = xdbOpenIndex(IDXname, dbHandle)
 
   msg.s = "DBFname: " + DBFname + Chr(10) + "dbHandle:" +  Str(dbHandle) + Chr(10) + Chr(10) + "IDXname: " + IDXname + Chr(10) + "idxHandle:" + Str(idxHandle)
   MessageRequester("", msg, 0)

; Add records to the database and index
  NumRecs.l = 500

 Msg.s = "Press 'Okay' to create " + Str(NumRecs) + " database and index records."
    Msg.s  ="Completed"
    MessageRequester(Title, msg, 0)
    
; ------------------------------------------------------------------------
; Predefine the field#'s for speed. If you already know the
; which field name corresponds to which field number then
; there is no need to do this. These functions provide
; really fast lookups.
; ------------------------------------------------------------------------
  CustIDfield = xdbFieldNumber(dbHandle, "CUSTID")
  CustNameField = xdbFieldNumber(dbHandle, "CUSTNAME")
  SalaryField = xdbFieldNumber(dbHandle, "SALARY")
 

  
  For x.l = 1 To NumRecs

      xdbClearBuffer(dbHandle)  ;this will clear the record buffer
     ;NOTE: You must do this step to assure clean information
          
      CustID.s = RSet(Trim(Str(x)), 7)
           
      xdbAssignField(dbHandle, "", CustIDfield, CustID)
      xdbAssignField(dbHandle, "", CustNameField, "The big Cat " + Str(x))
      xdbAssignField(dbHandle, "", SalaryField, "-1234.50")
      
     ;Add to the end of the database (Append) & add the key to the index.
      xdbAddRecord(dbHandle)  
      
  Next x

msg = "Press 'Okay' to MOVE FORWARD all records without index."
MessageRequester(Title, msg, 0)
 
; ***** FORWARD MOVEMENTS *****
; 
; using 0 as the index handle disables the use of indexes for the database.
; traverse the database using the database's natural order

    xdbMoveFirst(dbHandle, 0)
               
   
;Note: If you are not using an index then it is acceptable To use the 
;following code instead.
   NumRecords = xdbRecordCount(dbHandle)
   
   For x = 1 To NumRecords
 
       xdbGetRecord(dbHandle, x)
       ;you would access the record's fields here by using
       ;the xdbFieldValue functions.
  
   Next
        

msg = "Press 'Okay' to MOVE FORWARD all records using the index."
MessageRequester(Title, msg, 0)

 
;Records will be returned based on the index's key order (UPPER(CUSTNUM))
  xdbMoveFirst(dbHandle, idxHandle)

Repeat
   xdbMoveNext(dbHandle, idxHandle)
Until xdbEOF(dbHandle)

; '***** BACKWARDS MOVEMENTS *****
; 
Msg = "Press 'Okay' to MOVE BACKWARDS all records without index."
MessageRequester(Title, msg, 0)


;Using 0 as the index handle disables the use of indexes for the database.
;traverse the database using the database's natural order
   xdbMoveLast(dbHandle, 0)


;Note: If you are not using an index then it is acceptable to use the 
;following code instead.
    NumRecords.l = xdbRecordCount(dbHandle)
    
     For x = NumRecords To 1 Step -1
       xdbGetRecord(dbHandle, x)
         ;you would access the record's fields here by using
         ;the xdbFieldValue functions.
     Next
    
Msg = "Press 'Okay' to MOVE BACKWARDS all records using the index."
MessageRequester(Title, msg, 0)

 
;Records will be returned based on the index's key order (UPPER(CUSTNUM))
    xdbMoveLast(dbHandle, idxHandle)
Repeat
   xdbMovePrev(dbHandle, idxHandle)
Until xdbBOF(dbHandle) = #XDBTRUE
 
;Close the database and related index
 xdbClose(dbHandle)                                 

    
MessageRequester("PureBasic","Database and Index records moved successfully.", 0)    
End

; ExecutableFormat=Windows
; CursorPosition=145
; FirstLine=106
; EOF