;*********************************************************************
;
;Purpose: Demo code showing performance of Cheetah's xdbSpeedAppend.
;
;Paul Squires (2000-2003)
;
;*********************************************************************
; 
;Structure for the timer information

   Title.s = "PureBasic Cheetah Database System"

;Procedure checks for Cheetah errors (see bottom of file)
;Declare.s GetTime()

;Define the names of the database & index

  DBFname.s = "Speed.dbf"
  IDXname.s = "Speed.idx"           

   msg.s = "This is a simple demo that creates a 10,000 record sample database and index" 
   msg.s = msg + Chr(10)  + "   It demonstrates Cheetah's speed at creating databases from scratch." 
   msg.s = msg + Chr(10)  + "                                   Do you wish to continue?"

  Result = MessageRequester(Title, msg,#PB_MessageRequester_YesNo) 
  If Result = 7   ; pressed No button (Result = 7)    
     End
  EndIf
   
; Define the structure of the databases
; the database is very simple (only one field!)
; the index is on a 5 character Postal Code field.
; 
;Build the file definition.
   AllFields.s = "POSTAL,C,5,0"

;Create the database
   xdbCreate(DBFname, AllFields)
   Debug xdbAnyErrors() ;Calls error procedure at bottom....
 
;Open the database (database must be open prior To creating index)
    dbHandle = xdbOpen(DBFname, "") ; no encryption
    Debug xdbAnyErrors() 
 
; Create the index (database must be open)
   IndexExpr.s = "POSTAL"   ;index is not Case sensitive
   Duplicates.l = #XDBTRUE 
   
   xdbCreateIndex(IDXname, dbHandle, IndexExpr, Duplicates)
   Debug xdbAnyErrors()

; Open the index
   idxHandle = xdbOpenIndex(IDXname, dbHandle)

; Add records to the database and index
   NumRecs.l = 10000
   SpeedOn.l = #XDBTRUE
 ; SpeedOn.l = #XDBFALSE
 
  MessageRequester(Title, "Press 'Okay' to create " + Str(NumRecs) + " database records.", 0)

; 'predefine the field#'s For speed. If you already know the
; 'which field name corresponds to which field number then
; 'there is no need to do this. These functions provide
; 'really fast lookups.
   PostalField = xdbFieldNumber(dbHandle, "POSTAL")
   Postal.s = Str(xdbFieldLength(dbHandle, PostalField))

If SpeedOn.l = #XDBTRUE
;   Turn on the buffered append writes
    xdbSpeedAppend(dbHandle, #XDBTRUE)
EndIf

StartTimer = ElapsedMilliseconds()
   For x.l = 1 To NumRecs.l
 
      xdbClearBuffer(dbHandle)  ;this will clear the record buffer
      xdbAssignField(dbHandle, "", PostalField, RSet(Str(x), 5))
 
;      Add to the end of the database (Append) & add the key to the index.
      xdbAppendRecord(dbHandle)

      Result = xdbError()
       If Result
         Msg.s =  "Error: " + Str(Result) + " adding database record."
         MessageRequester("PureBasic", msg, 0)
         xdbResetError()
         Goto ExitOut
      EndIf
   Next
; 
 If SpeedOn = #XDBTRUE
;   Make sure to turn off buffered append writes
;   so all records are written.
   xdbSpeedAppend(dbHandle, #XDBFALSE)
 EndIf
; 
 
EndTime = ElapsedMilliseconds()
ans.s = Str(EndTime - StartTimer)
ans = ans + " milliseconds"
 
      xdbReindex(dbHandle, idxHandle, 0) ; 0 = continue if duplicates found
 
ExitOut:
 
;Close the database And related index
    xdbClose(dbHandle)                                 
    Msg.s  ="Completed in: " + ans
    MessageRequester(Title, msg, 0)

End

; ExecutableFormat=Windows
; CursorPosition=65
; FirstLine=30
; EOF