;-----------------------------------------------------------------------------
;                  (c) 2005 Thomas (ts-soft) Schulz
;
;     This library is free software; you can redistribute it and/or
;     modify it under the terms of the GNU Lesser General Public
;     License as published by the Free Software Foundation; either
;     version 2.1 of the License, Or (at your option) any later version.
;
;     This library is distributed in the hope that it will be useful,
;     but WITHOUT ANY WARRANTY; without even the implied warranty of
;     MERCHANTABILITY Or FITNESS For A PARTICULAR PURPOSE.  See the GNU
;     Lesser General Public License For more details.
;-----------------------------------------------------------------------------

; Formatversion
#VERSION = 512 ; 2.0 = HighByte 2, LowByte 0

; HeaderOffset
#oVERSION        =  0
#oCreationDate   =  2
#oLastAccessDate =  6
#oRecords        = 10
#oSatzlaenge     = 14
#oCheck          = 18
#oReserve        = 19

#RFile_HeaderLenght = 20
#RFile_First    = -1
#RFile_Previous = -2
#RFile_Next     = -3
#RFile_Last     = -4
#RFile_Current  = -5

Interface IRFile
  Create(FileName.s, RecordLength.l)
  Open(FileName.s)
  Close()
  RecCount()
  Eof()
  Lof()
  RecordLength()
  MaxRecord()
  LastUpdate()
  CreationDate()
  Version()
  VersionStr.s()
  Put(RecordNumber.l, MemoryPointer.l)
  Get(RecordNumber.l, MemoryPointer.l)
  Append(MemoryPointer.l)
  Crypt(MemoryPointer.l, PassWord.s)
EndInterface

Structure IT_IRFile
  IT.l[SizeOf(IRFile)/4]
EndStructure

Structure IRFile_M
  *VT.l
  File.l
  FileName.s
  RecordLength.l
  Version.w
  CreationDate.l
  LastAccessDate.l
  Changed.l
  Records.l
  openFlag.l
  lastRecord.l
EndStructure

;{ Helpfunctions
Procedure RFileSetBit(Value.l, BitNumber.l, Flag.l)
  BitNumber = 1 << BitNumber
  If Flag
    ProcedureReturn Value | BitNumber
  Else
    ProcedureReturn (Value | BitNumber) & ~BitNumber
  EndIf
EndProcedure

Procedure RFileTestBit(Value.l, BitNumber.l)
  If Value & (1 << BitNumber)
    ProcedureReturn #True
  Else
    ProcedureReturn #False
  EndIf
EndProcedure

Procedure RFileRC4Mem(Mem.l, memLen.l, key.s)
  ; based on source from Pille (German-Forum)
  If Key
    Dim S.l(255)
    Dim K.l(255)
    I.l=0: j.l=0: t.l=0: x.l=0
    temp.l=0: y.l=0
    j = 1
    l.l          =Len(Key)
    *Sp.LONG      = @S()
    *KeyP.BYTE    = @Key
    For i = 0 To 255
      *Sp\l = i
      *Sp + 4
      If *KeyP\b = 0
        *KeyP = @Key
      EndIf
      K(i) = *KeyP\b
      *KeyP+1
    Next i
    j = 0
    For i = 0 To 255
      j = (j + S(i) + K(i)) & 255
      temp = S(i)
      S(i) = S(j)
      S(j) = temp
    Next i
    i = 0
    j = 0
    *Memm.BYTE = Mem
    For x = 0 To memLen-1
      i = (i+1) & 255
      j = (j + S(i)) & 255
      temp = S(i)
      S(i) = S(j)
      S(j) = temp
      t = (S(i) + (S(j) & 255)) & 255
      y = S(t)
      *Memm\b ! y
      *Memm + 1
    Next
  EndIf
  ProcedureReturn Mem
EndProcedure
;}

; Methoden, don't use it directly
Procedure RFile_Open(*this.IRFile_M, FileName.s)
  Protected check.b
  *this\FileName = FileName
  If FileSize(*this\FileName) < #RFile_HeaderLenght : ProcedureReturn 0 : EndIf
  *this\File = OpenFile(#PB_Any, *this\FileName)
  If *this\File
    *this\openFlag = #True
    FileSeek(#oVERSION)
    If ReadWord() <> 512
      CloseFile(*this\File)
      ProcedureReturn #False
    EndIf
    FileSeek(#oVERSION)
    *this\Version = ReadWord()
    FileSeek(#oCreationDate)
    *this\CreationDate = ReadLong()
    FileSeek(#oLastAccessDate)
    *this\LastAccessDate = ReadLong()
    FileSeek(#oSatzlaenge)
    *this\RecordLength = ReadLong()
    *this\Changed = #False
    FileSeek(#oCheck)
    check = ReadByte()
    If RFileTestBit(check, 0) = #False
      Records = FileSize(*this\FileName) - #RFile_HeaderLenght
      If Records > 0
        Records / *this\RecordLength
        *this\Records = Records -1
        FileSeek(#oRecords)
        WriteLong(*this\Records)
      EndIf
    EndIf
    check = RFileSetBit(check, 0, 0)
    FileSeek(#oCheck)
    WriteByte(check)
    FileSeek(#oRecords)
    *this\Records = ReadLong()
    *this\lastRecord = -1
    ProcedureReturn *this\File
  EndIf
  ProcedureReturn #False
EndProcedure

Procedure RFile_Create(*this.IRFile_M, FileName.s, RecordLength.l)
  *this\FileName = FileName
  *this\RecordLength = RecordLength
  *this\File = CreateFile(#PB_Any, FileName)
  If *this\File
    FileSeek(#oVERSION)
    WriteWord(#VERSION)
    FileSeek(#oCreationDate)
    WriteLong(Date())
    FileSeek(#oLastAccessDate)
    WriteLong(Date())
    FileSeek(#oRecords)
    WriteLong(-1)
    FileSeek(#oSatzlaenge)
    WriteLong(*this\RecordLength)
    FileSeek(#oCheck)
    WriteByte(0)
    FileSeek(#oReserve)
    WriteByte(0)
    CloseFile(*this\File)
    ProcedureReturn RFile_Open(*this.IRFile_M, FileName.s)
  EndIf
  ProcedureReturn #False
EndProcedure

Procedure RFile_Close(*this.IRFile_M)
  Protected check.b
  If *this\openFlag
    UseFile(*this\File)
    If *this\Changed
      FileSeek(#oLastAccessDate)
      WriteLong(Date())
    EndIf
    FileSeek(#oRecords)
    WriteLong(*this\Records)
    FileSeek(#oCheck)
    check = ReadByte()
    check = RFileSetBit(check, 0, 1)
    FileSeek(#oCheck)
    WriteByte(check)
    CloseFile(*this\File)
  EndIf
  ProcedureReturn FreeMemory(*this)
EndProcedure

Procedure RFile_RecCount(*this.IRFile_M)
  ProcedureReturn *this\Records
EndProcedure

Procedure RFile_Eof(*this.IRFile_M)
  If *this\openFlag
    If *this\lastRecord = *this\Records
      ProcedureReturn #True
    EndIf
  EndIf
  ProcedureReturn #False
EndProcedure

Procedure RFile_Lof(*this.IRFile_M)
  If *this\openFlag
    UseFile(*this\File)
    ProcedureReturn Lof()
  EndIf
  ProcedureReturn #False
EndProcedure

Procedure RFile_RecordLength(*this.IRFile_M)
  ProcedureReturn *this\RecordLength
EndProcedure

Procedure RFile_MaxRecord(*this.IRFile_M)
  If *this\openFlag
    ProcedureReturn (2147483647 - #RFile_HeaderLenght) / *this\RecordLength
  EndIf
  ProcedureReturn #False
EndProcedure

Procedure RFile_LastUpdate(*this.IRFile_M)
  ProcedureReturn *this\LastAccessDate
EndProcedure

Procedure RFile_CreationDate(*this.IRFile_M)
  ProcedureReturn *this\CreationDate
EndProcedure

Procedure RFile_Version(*this.IRFile_M)
  If *this\openFlag
    UseFile(*this\File)
    FileSeek(#oVERSION)
    ProcedureReturn ReadWord()
  EndIf
  ProcedureReturn #False
EndProcedure

Procedure.s RFile_VersionStr(*this.IRFile_M)
  Protected Result.w
  If *this\openFlag
    UseFile(*this\File)
    FileSeek(#oVERSION)
    Result = ReadWord()
    ProcedureReturn "RFile-Version: " + Str((Result >> 8) & $FF) + "." + Str(Result & $FF)
  EndIf
  ProcedureReturn ""
EndProcedure

Procedure RFile_Put(*this.IRFile_M, RecordNumber.l, *MemoryPointer.l)
  If *this\openFlag
    UseFile(*this\File)
    If RecordNumber < 0
      Select RecordNumber
        Case #RFile_First
          RecordNumer = 0
        Case #RFile_Previous
          RecordNumber = *this\lastRecord -1
          If RecordNumber < 0 : ProcedureReturn #False : EndIf
        Case #RFile_Next
          RecordNumber = *this\lastRecord +1
          If RecordNumber > *this\Records : ProcedureReturn #False : EndIf
        Case #RFile_Last
          RecordNumber = *this\Records
        Case #RFile_Current
          RecordNumber = *this\lastRecord
        Default
          ProcedureReturn #False
      EndSelect
    EndIf
    If RecordNumber > *this\Records
      If Lof() + *this\RecordLength > 2147483647 : ProcedureReturn #False : EndIf
      *this\Records = RecordNumber
    EndIf
    FileSeek((*this\RecordLength * RecordNumber) + #RFile_HeaderLenght)
    WriteData(*MemoryPointer, *this\RecordLength)
    *this\Changed = #True
    *this\lastRecord = RecordNumber
    ProcedureReturn #True
  EndIf
  ProcedureReturn #False
EndProcedure

Procedure RFile_Get(*this.IRFile_M, RecordNumber.l, *MemoryPointer.l)
  If *this\openFlag
    UseFile(*this\File)
    If RecordNumber < 0
      Select RecordNumber
        Case #RFile_First
          RecordNumer = 0
        Case #RFile_Previous
          RecordNumber = *this\lastRecord -1
          If RecordNumber < 0 : ProcedureReturn #False : EndIf
        Case #RFile_Next
          RecordNumber = *this\lastRecord +1
          If RecordNumber > *this\Records : ProcedureReturn #False : EndIf
        Case #RFile_Last
          RecordNumber = *this\Records
        Case #RFile_Current
          RecordNumber = *this\lastRecord
        Default
          ProcedureReturn #False
      EndSelect
    EndIf
    If RecordNumber > *this\Records : ProcedureReturn #False : EndIf
    FileSeek((*this\RecordLength * RecordNumber) + #RFile_HeaderLenght)
    ReadData(*MemoryPointer, *this\RecordLength)
    *this\lastRecord = RecordNumber
    ProcedureReturn *MemoryPointer
  EndIf
  ProcedureReturn #False
EndProcedure

Procedure RFile_Append(*this.IRFile_M, *MemoryPointer.l)
  If *this\openFlag
    UseFile(*this\File)
    If Lof() + *this\RecordLength > 2147483647 : ProcedureReturn #False : EndIf
    *this\Records = *this\Records + 1
    FileSeek(*this\RecordLength * *this\Records + #RFile_HeaderLenght)
    WriteData(*MemoryPointer, *this\RecordLength)
    *this\Changed = #True
    *this\lastRecord = *this\Records
    ProcedureReturn *this\Records
  EndIf
  ProcedureReturn #False
EndProcedure

Procedure RFile_Crypt(*this.IRFile_M, *MemoryPointer.l, PassWord.s)
  If PassWord
    ProcedureReturn RFileRC4Mem(*MemoryPointer, *this\RecordLength, PassWord)
  EndIf
  ProcedureReturn *MemoryPointer
EndProcedure

; Objectcreation
ProcedureDLL New_RFile()
  Protected *object.IRFile_M
  Static *VT.IT_IRFile
  If *VT       = #Null
    *VT        = AllocateMemory(SizeOf(IT_IRFile))
    *VT\IT[0]  = @RFile_Create()
    *VT\IT[1]  = @RFile_Open()
    *VT\IT[2]  = @RFile_Close()
    *VT\IT[3]  = @RFile_RecCount()
    *VT\IT[4]  = @RFile_Eof()
    *VT\IT[5]  = @RFile_Lof()
    *VT\IT[6]  = @RFile_RecordLength()
    *VT\IT[7]  = @RFile_MaxRecord()
    *VT\IT[8]  = @RFile_LastUpdate()
    *VT\IT[9]  = @RFile_CreationDate()
    *VT\IT[10] = @RFile_Version()
    *VT\IT[11] = @RFile_VersionStr()
    *VT\IT[12] = @RFile_Put()
    *VT\IT[13] = @RFile_Get()
    *VT\IT[14] = @RFile_Append()
    *VT\IT[15] = @RFile_Crypt()
  EndIf
  *object    = AllocateMemory(SizeOf(IRFile_M))
  *object\VT = *VT
  ProcedureReturn *object
EndProcedure

; IDE Options = PureBasic v3.94 (Windows - x86)
; EnableXP
; DisableDebugger
; HideErrorLog