Global LastMessage.s

; Declarations:
Declare.s GetTableNames(lDataBaseHandle.l)               ; Returns then tablenames of the specified database
Declare.s GetIndices(sTablename.s, lDataBaseHandle.l)    ; Returns the indices of a table
Declare.s GetSchema(sTablename.s, lDataBaseHandle.l)     ; Returns a schema of a table
Declare.s GetFieldNames(sTablename.s, lDataBaseHandle.l) ; Returns the fieldnames of a table
Declare.l DumpExport(sDataBase.s)                        ; Exports a dump from a Database (by M. Heuertz)
Declare.l DumpImport(sDumpFileName.s,sDataBase.s)        ; Imports a dump into a new database (by M. Heuertz)
Declare.s GetFilenameWithoutExtension(sFile.s)           ; a helperfunction
 
Procedure.s GetTableNames(lDataBaseHandle.l)             ; Returns then tablenames of the specified database
  
  DefType.SQLite3_Recordset myRS
  DefType.s sSQL, sTablename
  
  If lDataBaseHandle = 0
    LastMessage = "Wrong Database-Handle"
    ProcedureReturn ""
  EndIf
  
  sSQL = "Select tbl_name From sqlite_master Where type='table' Order By tbl_name;"
  
  If SQLite3_GetRecordset(sSQL, lDataBaseHandle, @myRS )
    If myRS\Handle 
      While myRS\EOF=0
        If SQLite3_GetRecordsetValueByIndex(0, @myRS)
          sTablename + myRS\sValue + ";"
        EndIf
        SQLite3_RecordsetMoveNext(@myRS)
      Wend
    EndIf
  EndIf
  
  SQLite3_ReleaseRecordset(@myRS)
  
  If Right(sTablename,1)=";"
    sTablename = Left(sTablename,Len(sTablename)-1)
  EndIf
  
  ProcedureReturn sTablename
  
EndProcedure
Procedure.s GetIndices(sTablename.s, lDataBaseHandle.l)  ; Returns the indices of a table
  
  DefType.SQLite3_Recordset myRS
  DefType.s sSQL, sIndices
  
  If lDataBaseHandle = 0
    LastMessage = "Wrong Database-Handle"
    ProcedureReturn ""
  EndIf
  
  If sTablename = ""
    LastMessage = "Tablename is empty"
    ProcedureReturn ""
  EndIf
  
  sSQL + "Select * from sqlite_master where type = 'index' and tbl_name = '" + sTablename + "'"
  
  If SQLite3_GetRecordset(sSQL, lDataBaseHandle, @myRS )
    If myRS\Handle 
      While myRS\EOF=0
        If SQLite3_GetRecordsetValueByIndex(1, @myRS)
          sIndices + myRS\sValue + ";"
        EndIf
        SQLite3_RecordsetMoveNext(@myRS)
      Wend
    EndIf
  EndIf
  
  SQLite3_ReleaseRecordset(@myRS)
  
  If Right(sIndices,1)=";"
    sIndices = Left(sIndices,Len(sIndices)-1)
  EndIf
  
  ProcedureReturn sIndices
  
EndProcedure
Procedure.s GetSchema(sTablename.s, lDataBaseHandle.l)   ; Returns a schema of a table
  
  DefType.SQLite3_Recordset myRS
  DefType.s sSQL, sSchema
  
  If lDataBaseHandle = 0
    LastMessage = "Wrong Database-Handle"
    ProcedureReturn ""
  EndIf
  
  If sTablename = ""
    LastMessage = "Tablename is empty"
    ProcedureReturn ""
  EndIf
  
  sSQL + "SELECT sql FROM "
  sSQL + "(SELECT * FROM sqlite_master UNION ALL SELECT * FROM sqlite_temp_master) "
  sSQL + "WHERE tbl_name = '" + sTablename + "' AND type!='meta' "
  sSQL + "ORDER BY type DESC, name "
  
  If SQLite3_GetRecordset(sSQL, lDataBaseHandle, @myRS )
    If myRS\Handle 
      If SQLite3_GetRecordsetValueByIndex(0, @myRS)
        sSchema + myRS\sValue 
      EndIf
    EndIf
  EndIf
  
  SQLite3_ReleaseRecordset(@myRS)
  
  ProcedureReturn sSchema
  
EndProcedure
Procedure.s GetFieldNames(sTablename.s, lDataBaseHandle.l) ; Returns the fieldnames of a table
  
  DefType.SQLite3_Recordset myRS
  DefType.s sSQL, sSchema, sFieldnames, sDummy1, sDummy2
  
  If lDataBaseHandle = 0
    LastMessage = "Wrong Database-Handle"
    ProcedureReturn ""
  EndIf
  
  If sTablename = ""
    LastMessage = "Tablename is empty"
    ProcedureReturn ""
  EndIf
  
  sSchema.s = GetSchema(sTablename, lDataBaseHandle)
  
  lP1 = FindString(sSchema, "(",1)
  If lP1
    lP1 + 1
    lP2 = FindString(sSchema, ")",lP1)
    If lP2 
      sDummy1 = Mid(sSchema, lP1, lP2-lP1)
      lP1 = CountString(sDummy1, ",")+1
      For intI = 1 To lP1
        sDummy2 = Trim(StringField(sDummy1,intI, ","))
        If FindString(sDummy2," ",1)
          sFieldnames + StringField(sDummy2,1, " ") 
        Else
          sFieldnames + sDummy2 
        EndIf
        If intI <> lP1
          sFieldnames + ";"
        EndIf
      Next intI
    EndIf
  EndIf  
  
  ProcedureReturn sFieldnames
  
EndProcedure
Procedure.l DumpExport(sDataBase.s)                      ; Exports a dump from a Database (by M. Heuertz)
  
  ; Create a dumpfile named: [sDataBase]_[datum].sql
  ; Vielen Dank an M. Heuertz, von dem die Routine stammt!
  
  FF = CreateFile(#PB_Any,GetFilenameWithoutExtension(sDataBase)+"_"+FormatDate("%yyyy-%mm-%dd-%HH-%ii-%ss",Date())+".sql")
  
  If FF
    
    WriteString("BEGIN TRANSACTION;"+#CRLF$)
    
    Dim tablename.s(100)
    
    dbHandler = SQLite3_OpenDatabase(sDataBase) 
    
    If dbHandler
      ;
      ; 1. Alle Tabellennamen abfragen
      ;
      sSQL.s="SELECT name FROM sqlite_master WHERE type='table'"
      
      DefType.SQLite3_Recordset RS
      If SQLite3_GetRecordset(sSQL, dbHandler, @RS) 
        tablemax.l=RS\Rows
        SQLite3_RecordsetMoveFirst(@RS)
        While RS\EOF = 0
          If SQLite3_GetRecordsetValueByName("name", @RS)
            nbr.l=RS\CurrentPos-1
            tablename.s(nbr)=RS\sValue
          EndIf        
          SQLite3_RecordsetMoveNext(@RS)
        Wend
        SQLite3_ReleaseRecordset(@RS)
      EndIf
      ;
      ; 2. Pro Tabelle das Erstellungs-SQL abfragen und ins file schreiben
      ;
      For i=0 To tablemax-1
        
        DefType.SQLite3_Recordset RS
        
        sSQL.s="Select sql FROM sqlite_master WHERE name='"+tablename(i)+"'"  
        If SQLite3_GetRecordset(sSQL, dbHandler, @RS) 
          If RS\Handle
            If SQLite3_GetRecordsetValueByName("sql", @RS)
              sErgebnis.s = RS\sValue + #CRLF$ 
            EndIf
            SQLite3_ReleaseRecordset(@RS)
          EndIf
        EndIf
        
        If Right(sErgebnis, 2)=#CRLF$ : sErgebnis = Left(sErgebnis,Len(sErgebnis)-2) : EndIf
        WriteString(sErgebnis + ";"+ #CRLF$)
        
        ; 3. Dann die Daten der entsprechenden Tabelle abfragen und ins file zufgen
        ;
        If SQLite3_GetRecordset("SELECT * FROM "+tablename(i), dbHandler,@RS) 
          If RS\Handle
            SQLite3_RecordsetMoveFirst(@RS)
            While RS\EOF = 0
              imax=RS\Cols
              sErgebnis.s=""
              For n=0 To imax-1
                If SQLite3_GetRecordsetValueByIndex(n,@RS)
                  If Len(sErgebnis)>0
                    sErgebnis = sErgebnis + ",'" + RS\sValue +"'"
                  Else 
                    sErgebnis = "'" + RS\sValue + "'"
                  EndIf 
                EndIf
              Next
              If sErgebnis <> ""
                sErgebnis="INSERT INTO " + tablename(i) + " VALUES(" + sErgebnis + ");" + #CRLF$ 
                WriteString(sErgebnis)
              EndIf
              SQLite3_RecordsetMoveNext(@RS)
            Wend
            SQLite3_ReleaseRecordset(@RS)  
          EndIf
        EndIf     
      Next
      ;
      ; 4. Alle verbleibenden Objekte wie Views, Triggers und Indexe abfragen
      ;
      sSQL.s="SELECT name FROM sqlite_master WHERE sql>0 and type<>'table'"
      DefType.SQLite3_Recordset RS
      If SQLite3_GetRecordset(sSQL, dbHandler, @RS) 
        tablemax.l=RS\Rows
        SQLite3_RecordsetMoveFirst(@RS)
        While RS\EOF = 0
          If SQLite3_GetRecordsetValueByName("name", @RS)
            nbr.l=RS\CurrentPos-1
            tablename.s(nbr)=RS\sValue
          EndIf        
          SQLite3_RecordsetMoveNext(@RS)
        Wend
        SQLite3_ReleaseRecordset(@RS)
      EndIf
      ;
      ; 5. Pro Objekt das Erstellungs-SQL abfragen und ins file schreiben
      ;
      For i=0 To tablemax-1
        DefType.SQLite3_Recordset RS
        sSQL.s="Select sql FROM sqlite_master WHERE name='"+tablename(i)+"'"  
        If SQLite3_GetRecordset(sSQL, dbHandler, @RS) 
          If RS\Handle
            If SQLite3_GetRecordsetValueByName("sql", @RS)
              sErgebnis.s = RS\sValue + #CRLF$ 
            EndIf
            SQLite3_ReleaseRecordset(@RS)
          EndIf
        EndIf
        If Right(sErgebnis, 2)=#CRLF$ : sErgebnis = Left(sErgebnis,Len(sErgebnis)-2) : EndIf
        WriteString(sErgebnis+";"+ #CRLF$)
      Next
    EndIf
    WriteString("COMMIT;")
    CloseFile(FF)
    
    LastMessage = "OK"
    ProcedureReturn #True
    
  Else
    
    LastMessage = "Couldn't dump database: '" + sDataBase + "'"
    ProcedureReturn #False
    
  EndIf
  
EndProcedure 
Procedure.l DumpImport(sDumpFileName.s,sDataBase.s)      ; Imports a dump into a new database (by M. Heuertz)
  ;
  ; Create a database named 'dbfilename' (if not exists)
  ; read the Dump named 'dumpfilename'.
  ; Vielen Dank an M. Heuertz, von dem die Routine stammt!
  ;
  If (FileSize(sDataBase))=-1  ; Test ob die Datenbank existiert
    If FileSize(sDumpFileName) <> 0
      dbHandle = SQLite3_CreateDatabase(sDataBase,1)
      If dbHandle 
        FF = ReadFile(#PB_Any,sDumpFileName) 
        If FF
          Repeat
            sSQL.s=""
            Repeat
              sSQL = sSQL + ReadString()
            Until FindString(sSQL,";",1)>1  
            SQLite3_Execute(sSQL, dbHandle) 
          Until Eof(FF)
          CloseFile(FF)
          SQLite3_CloseDatabase(dbHandle) 
          LastMessage = "OK"
          ProcedureReturn #True ; Operation erfolgreich
        Else
          SQLite3_CloseDatabase(dbHandle) 
          DeleteFile(sDataBase)
          LastMessage = "Couldn't open DumpFile '" + sDumpFileName + "'."
          ProcedureReturn #False ; Dump nicht vorhanden, also keine Aktion
        EndIf 
      EndIf
    Else
      LastMessage = "DumpFile '" + sDumpFileName + "' doesn't exists."
      ProcedureReturn #False ; Dump nicht vorhanden, also keine Aktion
    EndIf
  Else
    LastMessage = "Database '" + sDataBase + "' already exists."
    ProcedureReturn #False ; DB exisiert, also keine Aktion
  EndIf   
  
EndProcedure  
Procedure.s GetFilenameWithoutExtension(sFile.s)         ; a little helperfunction
  
  ; Separate filename and suffix:
  For iPos = Len(sFile) To 1 Step -1
    If Mid(sFile, iPos, 1) = "."
      Break
    EndIf
  Next iPos
  
  If iPos 
    ProcedureReturn Left(sFile, iPos - 1)
  Else
    ProcedureReturn sFile
  EndIf
  
EndProcedure

; Init lib
If SQLite3_InitLib("sqlite3.dll") = #False 
  MessageRequester("PBOSL_SQLite3 (usefull_functions)", "Couldn't initialize sqlite3.dll")
  End
EndIf

; Create database
dbHandler = SQLite3_CreateDatabase("usefull_functions.db", #True)
  
If dbHandler = 0 
  MessageRequester("PBOSL_SQLite3 (usefull_functions)", "Couldn't create database 'usefull_functions.db'")
  End
EndIf

; Create table tblTest
SQLite3_Execute("Create Table tblTest (fldInteger Integer, fldText Text)",dbHandler)

; Create an index on field fldInteger
SQLite3_Execute("Create Index myIndex On tblTest (fldInteger)",dbHandler)

; Call the functions above

Debug "Tablename(s): "              + GetTableNames(dbHandler)
Debug "Indice(s): "                 + GetIndices("tblTest", dbHandler)
Debug "Schema from tblTest: "       + GetSchema("tblTest", dbHandler)
Debug "Fieldname(s) from tblTest: " + GetFieldNames("tblTest", dbHandler)

SQLite3_CloseDatabase(dbHandler)

SQLite3_End()

End

; IDE Options = PureBasic v3.94 (Windows - x86)
; EnableXP
; HideErrorLog