;******************************************************************************************
;'egrid' editable grid control. By Stephen Rodriguez.
;******************************************************************************************

;                                 DEMONSTRATION PROGRAM 2.


;******************************************************************************************
;This demo program sets up an egrid which exhibits every kind of possible celltype.
;
;No colours, no validation of data entered, just a simple illustration of all celltypes.
;
;Using different celltypes requires the use of a simple CellCallback function 
;(see the user guide for details).
;******************************************************************************************


;***************************************MUST INCLUDE***************************************
;XIncludeFile "egrid_UserFunctions.pb" 
;******************************************************************************************

LoadImage(1,"test.bmp")
LoadFont(2, "Arial", 12)


;************************************CELLCALLBACK FUNCTION*********************************

;NOTE, THIS IS NOT A PROPER WINDOWS CALLBACK!

  Procedure.l MyCellCallBack(egrid, uMsg, *cellinfo.egridCellInfo)
    Protected result
    Select uMsg
      Case #egrid_NotifyCellType
        Select *cellinfo\column
          Case 1
            *cellinfo\celltype=#egrid_PBImage
            *cellinfo\param=1 ;Image 1.
            *cellinfo\transparentcolour = #White
          Case 2
            *cellinfo\celltype=#egrid_String ;This is the default anyhow.
          Case 3
            *cellinfo\celltype=#egrid_StaticString
          Case 4
            *cellinfo\celltype=#egrid_ComboBoxEdit
;The following line lists the contents of the list part of the combobox.
            *cellinfo\text="Sunday" + Chr(10) + "Monday" + Chr(10) + "Tuesday" + Chr(10) + "Wednesday"+ Chr(10) + "Thursday"+ Chr(10) + "Friday"+ Chr(10) + "Saturday"
          Case 5
            *cellinfo\celltype=#egrid_ComboBox
;The following line lists the contents of the list part of the combobox.
            *cellinfo\text="Sunday" + Chr(10) + "Monday" + Chr(10) + "Tuesday" + Chr(10) + "Wednesday"+ Chr(10) + "Thursday"+ Chr(10) + "Friday"+ Chr(10) + "Saturday"
          Case 6
            *cellinfo\celltype=#egrid_CheckBox
;The following line lists the boolean values being used. You can use any pair of values, e.g. "1/0", "yes, no" etc.
;The first field is regarded as 'true'.
            *cellinfo\text="TRUE/FALSE"
          Case 7
            *cellinfo\celltype=#egrid_StaticCheckBox
;The following line lists the boolean values being used. You can use any pair of values, e.g. "1/0", "yes, no" etc.
;The first field is regarded as 'true'.
            *cellinfo\text="TRUE/FALSE"
        EndSelect
      Default 
        result = #True ;This accounts for all messages we have decided not to process.
    EndSelect
    ProcedureReturn result
  EndProcedure
;******************************************************************************************


;*************************************WINDOW + GADGET LIST*********************************
If OpenWindow(0,0,0,640,300,#PB_Window_SystemMenu|#PB_Window_MinimizeGadget|#PB_Window_MaximizeGadget|#PB_Window_Invisible|#PB_Window_ScreenCentered,"egrids test program.") And CreateGadgetList(WindowID(0))
  ShowWindow_(WindowID(), #SW_MAXIMIZE)
;******************************************************************************************


;********************************************EGRID*****************************************
;Create an egrid with resizable columns.
  egrid_CreateGrid(1, WindowWidth()/4, WindowHeight()/8, WindowWidth()/2, WindowHeight()/2,21,0, #egrid_ResizeColumnsTrue)
;******************************************************************************************


;**********************************SET CELLCALLBACK FUNCTION*******************************
;Set the callback early to avoid massive flickering when populating the egrid with initial data.
  egrid_CreateCellCallback(1, @MyCellCallBack())
;******************************************************************************************


;********************************************TEXT GADGET***********************************
  TextGadget(2,40, 21*WindowHeight()/32, WindowWidth()-80, WindowHeight()/4, "")
  SetGadgetFont(2,UseFont(2))
  text$="Cells in column 1 have a transparent image -white has been declared as transparent in this case."
  text$+Chr(10)+Chr(10)+"Column 2 has cells which are normal (default) string types."
  text$+Chr(10)+Chr(10)+"Cells in columns 3 and 7 are static, which means they can be selected but not altered in any way."
  
  SetGadgetText(2, text$)

;***************************************ADD DATA TO EGRID**********************************
  AddGadgetColumn(1,1,"Transparent Image",50)
  AddGadgetColumn(1,2,"String",100)
  AddGadgetColumn(1,3,"Static string",100)
  AddGadgetColumn(1,4,"Editable combo box",100)
  AddGadgetColumn(1,5,"Combo box",100)
  AddGadgetColumn(1,6,"Check box",50)
  AddGadgetColumn(1,7,"Static check box",50)
  
  For b=0 To 99; Add 100 rows.
    AddGadgetItem(1,-1,"") ;The "" is for the zero column which is always invisible in an egrid.
    SetGadgetItemText(1,b,"",1)
    SetGadgetItemText(1,b,"String "+Str(b),2)
    SetGadgetItemText(1,b,"Static "+Str(b),3)
    If b&1=1 ;Is b odd?
      SetGadgetItemText(1,b,"Saturday",4)
      SetGadgetItemText(1,b,"Sunday",5)
      SetGadgetItemText(1,b,"TRUE",6)
      SetGadgetItemText(1,b,"FALSE",7)
    Else
      SetGadgetItemText(1,b,"Monday",4)
      SetGadgetItemText(1,b,"Tuesday",5)
      SetGadgetItemText(1,b,"FALSE",6)
      SetGadgetItemText(1,b,"TRUE",7)
    EndIf
  Next
;NOTE that the zero column of an egrid is blanked out, hence the "" in the above AddGadgetItem() command.
;******************************************************************************************


;*****************************************EVENT LOOP***************************************
  Repeat
    EventID = WaitWindowEvent()
        
    Select EventID
     
      Case #PB_Event_Gadget
        Select EventGadgetID() 
      EndSelect
    EndSelect
  Until EventID = #PB_Event_CloseWindow
  
EndIf
  
End
;******************************************************************************************

; IDE Options = PureBasic v3.94 (Windows - x86)
; EnableAsm
; EnableXP
; Executable = test.exe