;******************************************************************************************
;'egrid' editable grid control. By Stephen Rodriguez.
;******************************************************************************************

;                                 DEMONSTRATION PROGRAM 3.


;******************************************************************************************
;This demo program sets up an egrid which exhibits cells of type 'String' only.
;
;No colours, no validation of data entered, just a simple illustration of all celltypes.
;
;Using different celltypes requires the use of a simple CellCallback function 
;(see the user guide for details).
;******************************************************************************************


;***************************************MUST INCLUDE***************************************
;XIncludeFile "egrid_UserFunctions.pb" 
;******************************************************************************************

LoadImage(1,"test.bmp")
LoadFont(2, "Arial", 12)


;************************************CELLCALLBACK FUNCTION*********************************

;NOTE, THIS IS NOT A PROPER WINDOWS CALLBACK!

  Procedure.l MyCellCallBack(egrid, uMsg, *cellinfo.egridCellInfo)
    Protected result
    Select uMsg


;This is called when the user is selecting a new cell. Here the behaviour observed in column 1 is controlled.
      Case #egrid_SelectCell
        result=#True
        If *cellinfo\column=1
          Select *cellinfo\param 
            Case #egrid_Tab
              egrid_MakeVisible(1,*cellinfo\row,1)
              *cellinfo\column=2
            Case #egrid_CtrlHome
              *cellinfo\row=0
              egrid_MakeVisible(1,*cellinfo\row,1)
              *cellinfo\column=2

            Default
              egrid_MakeVisible(1,*cellinfo\row,1)
              result=#False
          EndSelect
        EndIf


;This is called whenever a cell is being painted and/or font / colour information is required for
;the particular cell. 
      Case #egrid_FormatCell
        If *cellinfo\column = 1
          *cellinfo\backcolour=GetSysColor_(#COLOR_3DFACE)        
        ElseIf *cellinfo\row = egrid_SelectedRow(1) And *cellinfo\column = egrid_SelectedColumn(1)
          *cellinfo\backcolour = RGB(238, 247, 17)
        ElseIf *cellinfo\row = egrid_SelectedRow(1) Or *cellinfo\column = egrid_SelectedColumn(1)
          *cellinfo\backcolour = RGB(13, 249, 242)
        EndIf
    
;Use the default for any messages you decide not to process. Indeed you do not have to include code
;for all of the above messages.
      Default 
        result = #True
    EndSelect
    ProcedureReturn result
  EndProcedure
;******************************************************************************************


;*************************************WINDOW + GADGET LIST*********************************
If OpenWindow(0,0,0,640,300,#PB_Window_SystemMenu|#PB_Window_MinimizeGadget|#PB_Window_MaximizeGadget|#PB_Window_Invisible|#PB_Window_ScreenCentered,"egrids test program.") And CreateGadgetList(WindowID(0))
  ShowWindow_(WindowID(), #SW_MAXIMIZE)
;******************************************************************************************


;********************************************EGRID*****************************************
;Create an egrid with resizable columns.
  egrid_CreateGrid(1, WindowWidth()/4, WindowHeight()/8, 573, WindowHeight()/2,24,#PB_ListIcon_GridLines, #egrid_ResizeColumnsTrue)
;******************************************************************************************

;********************************************TEXT GADGET***********************************
  TextGadget(2,40, 21*WindowHeight()/32, WindowWidth()-80, WindowHeight()/4, "")
  SetGadgetFont(2,UseFont(2))
  text$="Some silliness with colours!"+Chr(10)+Chr(10)
  text$+"This is also supposed to look a little like a spreadsheet!"+Chr(10)+Chr(10)
  text$+"For the first column to act like labels requires the use of a simple CellCallback function."+Chr(10)
  text$+"Indeed, most formatting and validating of cells requires the developer to use such a function."
  
  SetGadgetText(2, text$)


;**********************************SET CELLCALLBACK FUNCTION*******************************
;Set the callback early to avoid massive flickering when populating the egrid with initial data.
  egrid_CreateCellCallback(1, @MyCellCallBack())
;******************************************************************************************


;***************************************ADD DATA TO EGRID**********************************
  AddGadgetColumn(1,1,"",50)
  For b = 1 To 10 ;10 columns
    AddGadgetColumn(1,b+1,Str(b),50)
  Next  
  For b=0 To 25; Add 100 rows.
    AddGadgetItem(1,-1,"") ;The "" is for the zero column which is always invisible in an egrid.
    SetGadgetItemText(1,b,Chr('A'+b),1)
  Next
;NOTE that the zero column of an egrid is blanked out, hence the "" in the above AddGadgetItem() command.
;******************************************************************************************


;*****************************************EVENT LOOP***************************************
  Repeat
    EventID = WaitWindowEvent()
        
    Select EventID
     
      Case #PB_Event_Gadget
        Select EventGadgetID() 
      EndSelect
    EndSelect
  Until EventID = #PB_Event_CloseWindow
  
EndIf
  
End
;******************************************************************************************

; IDE Options = PureBasic v3.94 (Windows - x86)
; EnableAsm
; EnableXP
; Executable = test.exe