;******************************************************************************************
;'egrid' editable grid control. By Stephen Rodriguez.
;******************************************************************************************

;                                 DEMONSTRATION PROGRAM 4.


;******************************************************************************************
;This demo program sets up a more complex egrid which exhibits every kind of possible celltype.
;The example shows multiple colours and fonts and validates data.
;
;It shows how cell selection can be controlled (see the first column)
;
;All this is controlled by the use of a simple CellCallback function (see the user guide for details).
;******************************************************************************************


;***************************************MUST INCLUDE***************************************
;XIncludeFile "egrid_UserFunctions.pb" 
;******************************************************************************************

LoadImage(1,"test.bmp")
LoadFont(1,"Arial", 12, #PB_Font_Bold|#PB_Font_Italic)
LoadFont(2, "Arial", 12)

;************************************CELLCALLBACK FUNCTION*********************************
;NOTE, THIS IS NOT A PROPER WINDOWS CALLBACK!

  Procedure.l MyCellCallBack(egrid, uMsg, *cellinfo.egridCellInfo)
    Protected result
    Static flag.b, rowtoshade
    Select uMsg

;This is called before a character is written to the underlying cell and can be used to validate
;user entry etc.
      Case #egrid_InsertChar
        If *cellinfo\column = 2
;Convert all characters to uppercase in column 2.
          *cellinfo\param=Asc(UCase(Chr(*cellinfo\param)))
        EndIf
        result = #True ;Allow the update to take place for any other column.

;This is called whenever a selection of characters is being deleted from a cell of either the
;'string' type or an editable combobox type. Here, the developer can refuse the deletion etc.
      Case #egrid_DeleteChar
        result = #True ;Allow the update to take place.

;The following is called whenever focus moves from one cell to another and the contents of the
;original cell have changed. Here we do nothing.
      Case #egrid_CellUpdated
        result=#True ;Allow the update.

;This is called when the user is selecting a new cell. Here the behaviour observed in column 1 is controlled.
      Case #egrid_SelectCell
        result=#True
        If *cellinfo\column=1
          Select *cellinfo\param 
            Case #egrid_Tab
              egrid_MakeVisible(1,*cellinfo\row,1)
              *cellinfo\column=2
            Case #egrid_CtrlHome
              *cellinfo\row=0
              egrid_MakeVisible(1,*cellinfo\row,1)
              *cellinfo\column=2
            Default
              egrid_MakeVisible(1,*cellinfo\row,1)
              result=#False
          EndSelect
        EndIf


;This is called whenever a user is attempting to resize a column within an egrid which has the 'resizecolumnsflag'
;set to '#egrid_ResizeColumnsTrue'
;This way we can refuse the resizing on certain columns etc.
      Case #egrid_UserResizeColumn
        result=#True ;Allow the resizing.
        If *cellinfo\column = 2;Refuse resizing of column 2.
          result=#False
        EndIf
      
;This is called to notify the type of cell.
      Case #egrid_NotifyCellType
        Select *cellinfo\column
          Case 1
            *cellinfo\celltype=#egrid_PBImage
            If *cellinfo\row&1=0 ;Place images in the even rows only.
              *cellinfo\param=1 ;Image 1.
              *cellinfo\transparentcolour = #White
            Else
              *cellinfo\param=-1 ;Signifies the image is blank.
            EndIf
          Case 2
            *cellinfo\celltype=#egrid_String ;This is the default anyhow.
          Case 3
            *cellinfo\celltype=#egrid_String
          Case 4
            *cellinfo\celltype=#egrid_String
          Case 5
            *cellinfo\celltype=#egrid_StaticString
          Case 6
            *cellinfo\celltype=#egrid_ComboBoxEdit
;The following line lists the contents of the list part of the combobox.
            *cellinfo\text="Sunday" + Chr(10) + "Monday" + Chr(10) + "Tuesday" + Chr(10) + "Wednesday"+ Chr(10) + "Thursday"+ Chr(10) + "Friday"+ Chr(10) + "Saturday"
          Case 7
            *cellinfo\celltype=#egrid_ComboBox
;The following line lists the contents of the list part of the combobox.
            *cellinfo\text="Sunday" + Chr(10) + "Monday" + Chr(10) + "Tuesday" + Chr(10) + "Wednesday"+ Chr(10) + "Thursday"+ Chr(10) + "Friday"+ Chr(10) + "Saturday"
          Case 8
            *cellinfo\celltype=#egrid_CheckBox
;The following line lists the boolean values being used. You can use any pair of values, e.g. "1/0", "yes, no" etc.
;The first field is regarded as 'true'.
            *cellinfo\text="TRUE/FALSE"
          Case 9
            *cellinfo\celltype=#egrid_StaticCheckBox
;The following line lists the boolean values being used. You can use any pair of values, e.g. "1/0", "yes, no" etc.
;The first field is regarded as 'true'.
            *cellinfo\text="TRUE/FALSE"
        EndSelect

;This is called whenever a cell is being painted and/or font / colour information is required for
;the particular cell. 
      Case #egrid_FormatCell
          *cellinfo\backcolour = $03FAFC
          *cellinfo\forecolour = RGB(0,0,$ff)
          If *cellinfo\column = 1 Or *cellinfo\column = 6 Or *cellinfo\column = 7 ;Shade these columns differently than the rest.
            *cellinfo\backcolour = RGB(0,$ea,$ea)
          ElseIf *cellinfo\column = 2
            *cellinfo\forecolour = #Black
            *cellinfo\textjustification = #egrid_LeftText ;The default anyhow!
            *cellinfo\font = UseFont(1)
          ElseIf *cellinfo\column = 3 Or *cellinfo\column = 5
            *cellinfo\textjustification = #egrid_CenterText
          ElseIf *cellinfo\column = 4
            *cellinfo\textjustification = #egrid_RightText
          EndIf

;Use the default for any messages you decide not to process. Indeed you do not have to include code
;for all of the above messages.
      Default 
        result = #True
    EndSelect
    ProcedureReturn result
  EndProcedure
;******************************************************************************************


;*************************************WINDOW + GADGET LIST*********************************
If OpenWindow(0,0,0,640,300,#PB_Window_SystemMenu|#PB_Window_MinimizeGadget|#PB_Window_MaximizeGadget|#PB_Window_Invisible|#PB_Window_ScreenCentered,"egrids test program.") And CreateGadgetList(WindowID(0))
  ShowWindow_(WindowID(), #SW_MAXIMIZE)
;******************************************************************************************

;********************************************EGRID*****************************************
;Create an egrid with resizable columns.
  egrid_CreateGrid(1, WindowWidth()/8, WindowHeight()/8, 3*WindowWidth()/4, WindowHeight()/2,21,0, #egrid_ResizeColumnsTrue)
;******************************************************************************************


;********************************************TEXT GADGET***********************************
  TextGadget(2,40, 21*WindowHeight()/32, WindowWidth()-80, WindowHeight()/4, "")
  SetGadgetFont(2,UseFont(2))
  text$="Cells in column 1 have a transparent image -white has been declared as transparent in this case. "
  text$+"Note again how cells in this column cannot be selected. This is controlled by use of a CellCallback function which allows the programmer "
  text$+"to monitor cell selection, validate data, colour cells... etc."
  text$+Chr(10)+Chr(10)+"Cells in column 2 are of normal (default) string types, but shows the input being validated in that characters are converted immediately to uppercase. The same can be done with editable comboboxes. "
  text$+Chr(10)+"NOTE also that only this column is being prevented from being resized. Try it! Again this is due to the Cellcallback function."
  text$+Chr(10)+Chr(10)+"Cells in columns 5 and 9 are static, which means they can be selected but not altered in any way."
  SetGadgetText(2, text$)


;**********************************SET CELLCALLBACK FUNCTION*******************************
;Set the callback early to avoid massive flickering when populating the egrid with initial data.
  egrid_CreateCellCallback(1, @MyCellCallBack())
;******************************************************************************************


;***************************************ADD DATA TO EGRID**********************************
  AddGadgetColumn(1,1,"Transparent Image",50)
  AddGadgetColumn(1,2,"Left string",100)
  AddGadgetColumn(1,3,"Center string",100)
  AddGadgetColumn(1,4,"Right string",100)
  AddGadgetColumn(1,5,"Static string",100)
  AddGadgetColumn(1,6,"Editable combo box",100)
  AddGadgetColumn(1,7,"Combo box",100)
  AddGadgetColumn(1,8,"Check box",50)
  AddGadgetColumn(1,9,"Static check box",50)
  
  For b=0 To 99; Add 100 rows.
    AddGadgetItem(1,-1,"") ;The "" is for the zero column which is always invisible in an egrid.
    SetGadgetItemText(1,b,"",1)
    SetGadgetItemText(1,b,"Left "+Str(b),2)
    SetGadgetItemText(1,b,"Center "+Str(b),3)
    SetGadgetItemText(1,b,"Right "+Str(b),4)
    SetGadgetItemText(1,b,"Static "+Str(b),5)
    If b&1=1 ;Is b odd?
      SetGadgetItemText(1,b,"Saturday",6)
      SetGadgetItemText(1,b,"Sunday",7)
      SetGadgetItemText(1,b,"TRUE",8)
      SetGadgetItemText(1,b,"FALSE",9)
    Else
      SetGadgetItemText(1,b,"Monday",6)
      SetGadgetItemText(1,b,"Tuesday",7)
      SetGadgetItemText(1,b,"FALSE",8)
      SetGadgetItemText(1,b,"TRUE",9)
    EndIf
  Next
;NOTE that the zero column of an egrid is blanked out, hence the "" in the above AddGadgetItem() command.
;******************************************************************************************


;*****************************************EVENT LOOP***************************************
  Repeat
    EventID = WaitWindowEvent()
        
    Select EventID
     
      Case #PB_Event_Gadget
        Select EventGadgetID() 
      EndSelect
    EndSelect
  Until EventID = #PB_Event_CloseWindow
  
EndIf
  
End
;******************************************************************************************

; IDE Options = PureBasic v3.94 (Windows - x86)
; EnableAsm
; EnableXP